package common.biome;

import java.util.Arrays;

import common.block.Block;
import common.block.BlockColored;
import common.block.BlockDirt;
import common.block.BlockSand;
import common.color.DyeColor;
import common.init.Blocks;
import common.material.Material;
import common.rng.PerlinGen;
import common.rng.Random;
import common.rng.WeightedList;
import common.util.BlockPos;
import common.world.State;
import common.world.WorldServer;
import common.worldgen.ChunkPrimer;
import common.worldgen.tree.WorldGenTree;

public class BiomeMesa extends Biome
{
    private final boolean bryce;
    private final boolean soil;
    
    private State[] layers;
    private long layerSeed;
    private PerlinGen baseBryceGen;
    private PerlinGen highBryceGen;
    private PerlinGen clayColorGen;

    public BiomeMesa(int id, boolean bryce, boolean soil)
    {
        super(id);
        this.bryce = bryce;
        this.soil = soil;
//        this.setDisableRain();
//        this.setTemperatureLegacy(2.0F).setHumidity(0.0F);
        this.setHumidity(0.0f);
//        this.mobs.clear();
        this.topBlock = Blocks.sand.getState().withProperty(BlockSand.VARIANT, BlockSand.EnumType.RED_SAND);
        this.fillerBlock = Blocks.stained_hardened_clay.getState();
        this.treesPerChunk = -999;
        this.deadBushPerChunk = 20;
        this.reedsPerChunk = 3;
        this.cactiPerChunk = 5;
        this.flowersPerChunk = 0;
//        this.mobs.clear();

        if (soil)
        {
            this.treesPerChunk = 5;
        }
    }
    
    protected void addMobs(WeightedList<RngSpawn> mobs) {
    }

    public WorldGenTree genBigTreeChance(Random rand)
    {
        return this.worldGeneratorTrees;
    }

    public int getFoliageColorAtPos(BlockPos pos)
    {
        return 10387789;
    }

    public int getGrassColorAtPos(BlockPos pos)
    {
        return 9470285;
    }

    public void decorate(WorldServer worldIn, Random rand, BlockPos pos)
    {
        super.decorate(worldIn, rand, pos);
    }

    public void genTerrainBlocks(WorldServer worldIn, Random rand, ChunkPrimer chunkPrimerIn, int x, int z, double noiseVal)
    {
        if (this.layers == null || this.layerSeed != worldIn.getSeed())
        {
            this.setupLayers(worldIn.getSeed());
        }

        if (this.baseBryceGen == null || this.highBryceGen == null || this.layerSeed != worldIn.getSeed())
        {
            Random random = new Random(this.layerSeed);
            this.baseBryceGen = new PerlinGen(random, 4);
            this.highBryceGen = new PerlinGen(random, 1);
        }

        this.layerSeed = worldIn.getSeed();
        double d4 = 0.0D;

        if (this.bryce)
        {
            int i = (x & -16) + (z & 15);
            int j = (z & -16) + (x & 15);
            double d0 = Math.min(Math.abs(noiseVal), this.baseBryceGen.generate((double)i * 0.25D, (double)j * 0.25D));

            if (d0 > 0.0D)
            {
                double d1 = 0.001953125D;
                double d2 = Math.abs(this.highBryceGen.generate((double)i * d1, (double)j * d1));
                d4 = d0 * d0 * 2.5D;
                double d3 = Math.ceil(d2 * 50.0D) + 14.0D;

                if (d4 > d3)
                {
                    d4 = d3;
                }

                d4 = d4 + 64.0D;
            }
        }

        int j1 = x & 15;
        int k1 = z & 15;
        int l1 = worldIn.getSeaLevel();
        State iblockstate = Blocks.stained_hardened_clay.getState();
        State iblockstate3 = this.fillerBlock;
        int k = (int)(noiseVal / 3.0D + 3.0D + rand.doublev() * 0.25D);
        boolean flag = Math.cos(noiseVal / 3.0D * Math.PI) > 0.0D;
        int l = -1;
        boolean flag1 = false;
        State worldState = worldIn.dimension.getFiller();
        Block worldBlock = worldState.getBlock();
        State worldAlt = worldIn.dimension.getAltFiller1();
        State liquid = worldIn.getSurfaceLiquid();
//        IBlockState bottom = worldIn.dimension.hasBedrock() ? Blocks.bedrock.getDefaultState() : Blocks.air.getDefaultState();

        for (int i1 = chunkPrimerIn.height - 1; i1 >= 0; --i1)
        {
            if (chunkPrimerIn.get(k1, i1, j1).getBlock().getMaterial() == Material.air && i1 < (int)d4)
            {
                chunkPrimerIn.set(k1, i1, j1, worldState);
            }

//            if (i1 <= rand.zrange(5) && bedrock)
//            {
//                chunkPrimerIn.set(k1, i1, j1, bottom);
//            }
//            else
//            {
            State iblockstate1 = chunkPrimerIn.get(k1, i1, j1);

            if (iblockstate1.getBlock().getMaterial() == Material.air)
            {
                l = -1;
            }
            else if (iblockstate1.getBlock() == worldBlock)
            {
                if (l == -1)
                {
                    flag1 = false;

                    if (k <= 0)
                    {
                        iblockstate = null;
                        iblockstate3 = worldState;
                    }
                    else if (i1 >= l1 - 4 && i1 <= l1 + 1)
                    {
                        iblockstate = Blocks.stained_hardened_clay.getState();
                        iblockstate3 = this.fillerBlock;
                    }

                    if (i1 < l1 && (iblockstate == null || iblockstate.getBlock().getMaterial() == Material.air))
                    {
                        iblockstate = liquid;
                    }

                    l = k + Math.max(0, i1 - l1);

                    if (i1 < l1 - 1)
                    {
                        chunkPrimerIn.set(k1, i1, j1, iblockstate3);

                        if (iblockstate3.getBlock() == Blocks.stained_hardened_clay)
                        {
                            chunkPrimerIn.set(k1, i1, j1, iblockstate3.getBlock().getState().withProperty(BlockColored.COLOR, DyeColor.ORANGE));
                        }
                    }
                    else if (this.soil && i1 > 86 + k * 2)
                    {
                        if (flag)
                        {
                            chunkPrimerIn.set(k1, i1, j1, Blocks.dirt.getState().withProperty(BlockDirt.VARIANT, BlockDirt.DirtType.COARSE_DIRT));
                        }
                        else
                        {
                            chunkPrimerIn.set(k1, i1, j1, Blocks.grass.getState());
                        }
                    }
                    else if (i1 <= l1 + 3 + k)
                    {
                        chunkPrimerIn.set(k1, i1, j1, this.topBlock);
                        flag1 = true;
                    }
                    else
                    {
                        State iblockstate4;

                        if (i1 >= 64 && i1 <= 127)
                        {
                            if (flag)
                            {
                                iblockstate4 = Blocks.hardened_clay.getState();
                            }
                            else
                            {
                                iblockstate4 = this.getBlockAt(x, i1, z);
                            }
                        }
                        else
                        {
                            iblockstate4 = Blocks.stained_hardened_clay.getState().withProperty(BlockColored.COLOR, DyeColor.ORANGE);
                        }

                        chunkPrimerIn.set(k1, i1, j1, iblockstate4);
                    }
                }
                else if (l > 0)
                {
                    --l;

                    if (flag1)
                    {
                        chunkPrimerIn.set(k1, i1, j1, Blocks.stained_hardened_clay.getState().withProperty(BlockColored.COLOR, DyeColor.ORANGE));
                    }
                    else
                    {
                        State iblockstate2 = this.getBlockAt(x, i1, z);
                        chunkPrimerIn.set(k1, i1, j1, iblockstate2);
                    }
                }
            }
        }
//        }
    }

    private void setupLayers(long seed)
    {
        this.layers = new State[64];
        Arrays.fill(this.layers, Blocks.hardened_clay.getState());
        Random random = new Random(seed);
        this.clayColorGen = new PerlinGen(random, 1);

        for (int l1 = 0; l1 < 64; ++l1)
        {
            l1 += random.roll(5);

            if (l1 < 64)
            {
                this.layers[l1] = Blocks.stained_hardened_clay.getState().withProperty(BlockColored.COLOR, DyeColor.ORANGE);
            }
        }

        int i2 = random.range(2, 5);

        for (int i = 0; i < i2; ++i)
        {
            int j = random.roll(3);
            int k = random.zrange(64);

            for (int l = 0; k + l < 64 && l < j; ++l)
            {
                this.layers[k + l] = Blocks.stained_hardened_clay.getState().withProperty(BlockColored.COLOR, DyeColor.YELLOW);
            }
        }

        int j2 = random.range(2, 5);

        for (int k2 = 0; k2 < j2; ++k2)
        {
            int i3 = random.range(2, 4);
            int l3 = random.zrange(64);

            for (int i1 = 0; l3 + i1 < 64 && i1 < i3; ++i1)
            {
                this.layers[l3 + i1] = Blocks.stained_hardened_clay.getState().withProperty(BlockColored.COLOR, DyeColor.BROWN);
            }
        }

        int l2 = random.range(2, 5);

        for (int j3 = 0; j3 < l2; ++j3)
        {
            int i4 = random.roll(3);
            int k4 = random.zrange(64);

            for (int j1 = 0; k4 + j1 < 64 && j1 < i4; ++j1)
            {
                this.layers[k4 + j1] = Blocks.stained_hardened_clay.getState().withProperty(BlockColored.COLOR, DyeColor.RED);
            }
        }

        int k3 = random.range(3, 5);
        int j4 = 0;

        for (int l4 = 0; l4 < k3; ++l4)
        {
            int i5 = 1;
            j4 += random.range(4, 19);

            for (int k1 = 0; j4 + k1 < 64 && k1 < i5; ++k1)
            {
                this.layers[j4 + k1] = Blocks.stained_hardened_clay.getState().withProperty(BlockColored.COLOR, DyeColor.WHITE);

                if (j4 + k1 > 1 && random.chance())
                {
                    this.layers[j4 + k1 - 1] = Blocks.stained_hardened_clay.getState().withProperty(BlockColored.COLOR, DyeColor.SILVER);
                }

                if (j4 + k1 < 63 && random.chance())
                {
                    this.layers[j4 + k1 + 1] = Blocks.stained_hardened_clay.getState().withProperty(BlockColored.COLOR, DyeColor.SILVER);
                }
            }
        }
    }

    private State getBlockAt(int x, int y, int z)
    {
        int i = (int)Math.round(this.clayColorGen.generate((double)x * 1.0D / 512.0D, (double)x * 1.0D / 512.0D) * 2.0D);
        return this.layers[(y + i + 64) % 64];
    }

    protected Biome createMutatedBiome(int id)
    {
        boolean bryce = this.id == Biome.mesa.id;
        BiomeMesa mesa = new BiomeMesa(id, bryce, this.soil);

        if (!bryce)
        {
            mesa.setScaling(Scaling.HILLS_LOW);
            mesa.setBiomeName(this.name + "M", this.display + " M");
        }
        else
        {
            mesa.setBiomeName(this.name + "Bryce", this.display + " (Bryce)");
        }

        mesa.setColor(this.color);
        return mesa;
    }
}
