package common.biome;

import common.block.BlockDoublePlant;
import common.block.BlockFlower;
import common.entity.animal.EntityHorse;
import common.rng.Random;
import common.util.BlockPos;
import common.world.WorldServer;

public class BiomePlains extends Biome
{
    private static final BlockFlower.EnumFlowerType[] TULIP_TYPES = new BlockFlower.EnumFlowerType[] {
    		BlockFlower.EnumFlowerType.ORANGE_TULIP, BlockFlower.EnumFlowerType.RED_TULIP,
    		BlockFlower.EnumFlowerType.PINK_TULIP, BlockFlower.EnumFlowerType.WHITE_TULIP
    };

    private static final BlockFlower.EnumFlowerType[] FLOWER_TYPES = new BlockFlower.EnumFlowerType[] {
    		BlockFlower.EnumFlowerType.POPPY, BlockFlower.EnumFlowerType.HOUSTONIA, BlockFlower.EnumFlowerType.OXEYE_DAISY
    };
    
//    protected boolean field_150628_aC;

    protected BiomePlains(int id)
    {
        super(id);
        this.setTemperature(12.0f).setHumidity(40.0f);
        this.setScaling(Scaling.PLAINS_LOW);
        this.mobs.add(new RngSpawn(EntityHorse.class, 5, 2, 6));
        this.treesPerChunk = -999;
        this.flowersPerChunk = 4;
        this.grassPerChunk = 10;
    }

    public BlockFlower.EnumFlowerType pickRandomFlower(Random rand, BlockPos pos)
    {
        double d0 = GRASS_NOISE.generate((double)pos.getX() / 200.0D, (double)pos.getZ() / 200.0D);

        if (d0 < -0.8D)
        {
        	return rand.pick(TULIP_TYPES);
//            int j = rand.nextInt(4);
//
//            switch (j)
//            {
//                case 0:
//                    return BlockFlower.EnumFlowerType.ORANGE_TULIP;
//
//                case 1:
//                    return BlockFlower.EnumFlowerType.RED_TULIP;
//
//                case 2:
//                    return BlockFlower.EnumFlowerType.PINK_TULIP;
//
//                case 3:
//                default:
//                    return BlockFlower.EnumFlowerType.WHITE_TULIP;
//            }
        }
        else if (rand.rarity(3))
        {
        	return rand.pick(FLOWER_TYPES);
//            int i = rand.nextInt(3);
//            return i == 0 ? BlockFlower.EnumFlowerType.POPPY : (i == 1 ? BlockFlower.EnumFlowerType.HOUSTONIA : BlockFlower.EnumFlowerType.OXEYE_DAISY);
        }
        else
        {
            return BlockFlower.EnumFlowerType.DANDELION;
        }
    }

    public void decorate(WorldServer worldIn, Random rand, BlockPos pos)
    {
        double d0 = GRASS_NOISE.generate((double)(pos.getX() + 8) / 200.0D, (double)(pos.getZ() + 8) / 200.0D);

        if (d0 < -0.8D)
        {
            this.flowersPerChunk = 15;
            this.grassPerChunk = 5;
        }
        else
        {
            this.flowersPerChunk = 4;
            this.grassPerChunk = 10;
            DOUBLE_PLANT_GEN.setPlantType(BlockDoublePlant.EnumPlantType.GRASS);

            for (int i = 0; i < 7; ++i)
            {
                int j = rand.chOffset();
                int k = rand.chOffset();
                int l = rand.zrange(worldIn.getHeight(pos.add(j, 0, k)).getY() + 32);
                DOUBLE_PLANT_GEN.generate(worldIn, rand, pos.add(j, l, k));
            }
        }

//        int n = rand.range(0, 2);
        if (rand.chance())
        {
            DOUBLE_PLANT_GEN.setPlantType(BlockDoublePlant.EnumPlantType.SUNFLOWER);

//            for (int i1 = 0; i1 < 10; ++i1)
//            {
            int j1 = rand.chOffset();
            int k1 = rand.chOffset();
            int l1 = rand.zrange(worldIn.getHeight(pos.add(j1, 0, k1)).getY() + 32);
            DOUBLE_PLANT_GEN.generate(worldIn, rand, pos.add(j1, l1, k1));
//            }
        }

        super.decorate(worldIn, rand, pos);
    }

//    protected Biome createMutatedBiome(int p_180277_1_)
//    {
//        BiomePlains biomegenplains = new BiomePlains(p_180277_1_);
//        biomegenplains.setBiomeName("sunflowerPlains");
//        biomegenplains.field_150628_aC = true;
//        biomegenplains.setColor(9286496);
//        return biomegenplains;
//    }
}
