package common.entity.animal;

import java.util.List;

import common.collect.Lists;
import common.entity.DamageSource;
import common.entity.Entity;
import common.entity.EntityType;
import common.entity.npc.Alignment;
import common.entity.npc.EntityNPC;
import common.entity.types.EntityLiving;
import common.entity.types.IEntityMultiPart;
import common.init.Config;
import common.init.SoundEvent;
import common.model.ParticleType;
import common.util.ExtMath;
import common.util.Vec3;
import common.world.World;
import common.world.WorldClient;

public class EntityDragon extends EntityLiving implements IEntityMultiPart
{
    public double targetX;
    public double targetY;
    public double targetZ;
    public double[][] ringBuffer = new double[64][3];
    public int ringBufferIndex = -1;
    public EntityDragonPart[] dragonPartArray;
    public EntityDragonPart dragonPartHead;
    public EntityDragonPart dragonPartBody;
    public EntityDragonPart dragonPartTail1;
    public EntityDragonPart dragonPartTail2;
    public EntityDragonPart dragonPartTail3;
    public EntityDragonPart dragonPartWing1;
    public EntityDragonPart dragonPartWing2;
    public float prevAnimTime;
    public float animTime;
    public boolean forceNewTarget;
    private Entity target;

    public EntityDragon(World worldIn)
    {
        super(worldIn);
        this.dragonPartArray = new EntityDragonPart[] {this.dragonPartHead = new EntityDragonPart(this, "head", 6.0F, 6.0F), this.dragonPartBody = new EntityDragonPart(this, "body", 8.0F, 8.0F), this.dragonPartTail1 = new EntityDragonPart(this, "tail", 4.0F, 4.0F), this.dragonPartTail2 = new EntityDragonPart(this, "tail", 4.0F, 4.0F), this.dragonPartTail3 = new EntityDragonPart(this, "tail", 4.0F, 4.0F), this.dragonPartWing1 = new EntityDragonPart(this, "wing", 4.0F, 4.0F), this.dragonPartWing2 = new EntityDragonPart(this, "wing", 4.0F, 4.0F)};
        this.setHealth(this.getMaxHealth());
        this.setSize(16.0F, 8.0F);
        this.noClip = true;
//        this.fireImmune = true;
        this.targetY = 100.0D;
        this.noFrustumCheck = true;
        this.xpValue = 12000;
    }

    public boolean isImmuneToFire()
    {
        return true;
    }

    protected void applyEntityAttributes()
    {
        super.applyEntityAttributes();
        this.setMaxHealth(200);
    }

//    protected void entityInit()
//    {
//        super.entityInit();
//    }

	public boolean allowLeashing() {
		return false;
	}

    /**
     * Returns a double[3] array with movement offsets, used to calculate trailing tail/neck positions. [0] = yaw
     * offset, [1] = y offset, [2] = unused, always 0. Parameters: buffer index offset, partial ticks.
     */
    public double[] getMovementOffsets(int p_70974_1_, float p_70974_2_)
    {
        if (this.getHealth() <= 0)
        {
            p_70974_2_ = 0.0F;
        }

        p_70974_2_ = 1.0F - p_70974_2_;
        int i = this.ringBufferIndex - p_70974_1_ * 1 & 63;
        int j = this.ringBufferIndex - p_70974_1_ * 1 - 1 & 63;
        double[] adouble = new double[3];
        double d0 = this.ringBuffer[i][0];
        double d1 = ExtMath.wrapd(this.ringBuffer[j][0] - d0);
        adouble[0] = d0 + d1 * (double)p_70974_2_;
        d0 = this.ringBuffer[i][1];
        d1 = this.ringBuffer[j][1] - d0;
        adouble[1] = d0 + d1 * (double)p_70974_2_;
        adouble[2] = this.ringBuffer[i][2] + (this.ringBuffer[j][2] - this.ringBuffer[i][2]) * (double)p_70974_2_;
        return adouble;
    }
    
    public void onLivingUpdate()
    {
        if (this.worldObj.client)
        {
            float f = ExtMath.cos(this.animTime * (float)Math.PI * 2.0F);
            float f1 = ExtMath.cos(this.prevAnimTime * (float)Math.PI * 2.0F);

            if (f1 <= -0.3F && f >= -0.3F) // && !this.isSilent())
            {
            	((WorldClient)this.worldObj).playSound(this.posX, this.posY, this.posZ, SoundEvent.DRAGON_WINGS, 5.0F);
            }
        }

        this.prevAnimTime = this.animTime;

        if (this.getHealth() <= 0)
        {
            float f11 = (this.rand.floatv() - 0.5F) * 8.0F;
            float f13 = (this.rand.floatv() - 0.5F) * 4.0F;
            float f14 = (this.rand.floatv() - 0.5F) * 8.0F;
            this.worldObj.spawnParticle(ParticleType.EXPLOSION_LARGE, this.posX + (double)f11, this.posY + 2.0D + (double)f13, this.posZ + (double)f14, 0.0D, 0.0D, 0.0D);
        }
        else
        {
//            this.updateDragonEnderCrystal();
            float f10 = 0.2F / (ExtMath.sqrtd(this.motionX * this.motionX + this.motionZ * this.motionZ) * 10.0F + 1.0F);
            f10 = f10 * (float)Math.pow(2.0D, this.motionY);

            this.animTime += f10;

            this.rotYaw = ExtMath.wrapf(this.rotYaw);

//            if (this.isAIDisabled() || 
            if(!this.worldObj.client && !Config.mobTick) // )
            {
                this.animTime = 0.5F;
            }
            else
            {
                if (this.ringBufferIndex < 0)
                {
                    for (int i = 0; i < this.ringBuffer.length; ++i)
                    {
                        this.ringBuffer[i][0] = (double)this.rotYaw;
                        this.ringBuffer[i][1] = this.posY;
                    }
                }

                if (++this.ringBufferIndex == this.ringBuffer.length)
                {
                    this.ringBufferIndex = 0;
                }

                this.ringBuffer[this.ringBufferIndex][0] = (double)this.rotYaw;
                this.ringBuffer[this.ringBufferIndex][1] = this.posY;

                if (this.worldObj.client)
                {
                    if (this.moveIncrements > 0)
                    {
                        double d10 = this.posX + (this.newX - this.posX) / (double)this.moveIncrements;
                        double d0 = this.posY + (this.newY - this.posY) / (double)this.moveIncrements;
                        double d1 = this.posZ + (this.newZ - this.posZ) / (double)this.moveIncrements;
                        double d2 = ExtMath.wrapd(this.newYaw - (double)this.rotYaw);
                        this.rotYaw = (float)((double)this.rotYaw + d2 / (double)this.moveIncrements);
                        this.rotPitch = (float)((double)this.rotPitch + (this.newPitch - (double)this.rotPitch) / (double)this.moveIncrements);
                        --this.moveIncrements;
                        this.setPosition(d10, d0, d1);
                        this.setRotation(this.rotYaw, this.rotPitch);
                    }
                }
                else
                {
                    double d11 = this.targetX - this.posX;
                    double d12 = this.targetY - this.posY;
                    double d13 = this.targetZ - this.posZ;
                    double d14 = d11 * d11 + d12 * d12 + d13 * d13;

                    if (this.target != null)
                    {
                        this.targetX = this.target.posX;
                        this.targetZ = this.target.posZ;
                        double d3 = this.targetX - this.posX;
                        double d5 = this.targetZ - this.posZ;
                        double d7 = Math.sqrt(d3 * d3 + d5 * d5);
                        double d8 = 0.4000000059604645D + d7 / 80.0D - 1.0D;

                        if (d8 > 10.0D)
                        {
                            d8 = 10.0D;
                        }

                        this.targetY = this.target.getEntityBoundingBox().minY + d8;
                    }
                    else
                    {
                        this.targetX += this.rand.gaussian() * 2.0D;
                        this.targetZ += this.rand.gaussian() * 2.0D;
                    }

                    if (this.forceNewTarget || d14 < 100.0D || d14 > 22500.0D || this.collidedHorizontally || this.collidedVertically)
                    {
                        this.setNewTarget();
                    }

                    d12 = d12 / (double)ExtMath.sqrtd(d11 * d11 + d13 * d13);
                    float f17 = 0.6F;
                    d12 = ExtMath.clampd(d12, (double)(-f17), (double)f17);
                    this.motionY += d12 * 0.10000000149011612D;
                    this.rotYaw = ExtMath.wrapf(this.rotYaw);
                    double d4 = 180.0D - ExtMath.atan2(d11, d13) * 180.0D / Math.PI;
                    double d6 = ExtMath.wrapd(d4 - (double)this.rotYaw);

                    if (d6 > 50.0D)
                    {
                        d6 = 50.0D;
                    }

                    if (d6 < -50.0D)
                    {
                        d6 = -50.0D;
                    }

                    Vec3 vec3 = (new Vec3(this.targetX - this.posX, this.targetY - this.posY, this.targetZ - this.posZ)).normalize();
                    double d15 = (double)(-ExtMath.cos(this.rotYaw * (float)Math.PI / 180.0F));
                    Vec3 vec31 = (new Vec3((double)ExtMath.sin(this.rotYaw * (float)Math.PI / 180.0F), this.motionY, d15)).normalize();
                    float f5 = ((float)vec31.dotProduct(vec3) + 0.5F) / 1.5F;

                    if (f5 < 0.0F)
                    {
                        f5 = 0.0F;
                    }

                    this.randomYawVelo *= 0.8F;
                    float f6 = ExtMath.sqrtd(this.motionX * this.motionX + this.motionZ * this.motionZ) * 1.0F + 1.0F;
                    double d9 = Math.sqrt(this.motionX * this.motionX + this.motionZ * this.motionZ) * 1.0D + 1.0D;

                    if (d9 > 40.0D)
                    {
                        d9 = 40.0D;
                    }

                    this.randomYawVelo = (float)((double)this.randomYawVelo + d6 * (0.699999988079071D / d9 / (double)f6));
                    this.rotYaw += this.randomYawVelo * 0.1F;
                    float f7 = (float)(2.0D / (d9 + 1.0D));
                    float f8 = 0.06F;
                    this.moveFlying(0.0F, -1.0F, f8 * (f5 * f7 + (1.0F - f7)));

                    this.moveEntity(this.motionX, this.motionY, this.motionZ);

                    Vec3 vec32 = (new Vec3(this.motionX, this.motionY, this.motionZ)).normalize();
                    float f9 = ((float)vec32.dotProduct(vec31) + 1.0F) / 2.0F;
                    f9 = 0.8F + 0.15F * f9;
                    this.motionX *= (double)f9;
                    this.motionZ *= (double)f9;
                    this.motionY *= 0.9100000262260437D;
                }

                this.yawOffset = this.rotYaw;
                this.dragonPartHead.width = this.dragonPartHead.height = 3.0F;
                this.dragonPartTail1.width = this.dragonPartTail1.height = 2.0F;
                this.dragonPartTail2.width = this.dragonPartTail2.height = 2.0F;
                this.dragonPartTail3.width = this.dragonPartTail3.height = 2.0F;
                this.dragonPartBody.height = 3.0F;
                this.dragonPartBody.width = 5.0F;
                this.dragonPartWing1.height = 2.0F;
                this.dragonPartWing1.width = 4.0F;
                this.dragonPartWing2.height = 3.0F;
                this.dragonPartWing2.width = 4.0F;
                float f12 = (float)(this.getMovementOffsets(5, 1.0F)[1] - this.getMovementOffsets(10, 1.0F)[1]) * 10.0F / 180.0F * (float)Math.PI;
                float f2 = ExtMath.cos(f12);
                float f15 = -ExtMath.sin(f12);
                float f3 = this.rotYaw * (float)Math.PI / 180.0F;
                float f16 = ExtMath.sin(f3);
                float f4 = ExtMath.cos(f3);
                this.dragonPartBody.onUpdate();
                this.dragonPartBody.setLocationAndAngles(this.posX + (double)(f16 * 0.5F), this.posY, this.posZ - (double)(f4 * 0.5F), 0.0F, 0.0F);
                this.dragonPartWing1.onUpdate();
                this.dragonPartWing1.setLocationAndAngles(this.posX + (double)(f4 * 4.5F), this.posY + 2.0D, this.posZ + (double)(f16 * 4.5F), 0.0F, 0.0F);
                this.dragonPartWing2.onUpdate();
                this.dragonPartWing2.setLocationAndAngles(this.posX - (double)(f4 * 4.5F), this.posY + 2.0D, this.posZ - (double)(f16 * 4.5F), 0.0F, 0.0F);

                if (!this.worldObj.client && this.hurtTime == 0)
                {
                    this.collideWithEntities(this.worldObj.getEntitiesWithinAABBExcludingEntity(this, this.dragonPartWing1.getEntityBoundingBox().expand(4.0D, 2.0D, 4.0D).offset(0.0D, -2.0D, 0.0D)));
                    this.collideWithEntities(this.worldObj.getEntitiesWithinAABBExcludingEntity(this, this.dragonPartWing2.getEntityBoundingBox().expand(4.0D, 2.0D, 4.0D).offset(0.0D, -2.0D, 0.0D)));
                    if(Config.damageMobs)
                    	this.attackEntitiesInList(this.worldObj.getEntitiesWithinAABBExcludingEntity(this, this.dragonPartHead.getEntityBoundingBox().expand(1.0D, 1.0D, 1.0D)));
                }

                double[] adouble1 = this.getMovementOffsets(5, 1.0F);
                double[] adouble = this.getMovementOffsets(0, 1.0F);
                float f18 = ExtMath.sin(this.rotYaw * (float)Math.PI / 180.0F - this.randomYawVelo * 0.01F);
                float f19 = ExtMath.cos(this.rotYaw * (float)Math.PI / 180.0F - this.randomYawVelo * 0.01F);
                this.dragonPartHead.onUpdate();
                this.dragonPartHead.setLocationAndAngles(this.posX + (double)(f18 * 5.5F * f2), this.posY + (adouble[1] - adouble1[1]) * 1.0D + (double)(f15 * 5.5F), this.posZ - (double)(f19 * 5.5F * f2), 0.0F, 0.0F);

                for (int j = 0; j < 3; ++j)
                {
                    EntityDragonPart entitydragonpart = null;

                    if (j == 0)
                    {
                        entitydragonpart = this.dragonPartTail1;
                    }

                    if (j == 1)
                    {
                        entitydragonpart = this.dragonPartTail2;
                    }

                    if (j == 2)
                    {
                        entitydragonpart = this.dragonPartTail3;
                    }

                    double[] adouble2 = this.getMovementOffsets(12 + j * 2, 1.0F);
                    float f20 = this.rotYaw * (float)Math.PI / 180.0F + this.simplifyAngle(adouble2[0] - adouble1[0]) * (float)Math.PI / 180.0F * 1.0F;
                    float f21 = ExtMath.sin(f20);
                    float f22 = ExtMath.cos(f20);
                    float f23 = 1.5F;
                    float f24 = (float)(j + 1) * 2.0F;
                    entitydragonpart.onUpdate();
                    entitydragonpart.setLocationAndAngles(this.posX - (double)((f16 * f23 + f21 * f24) * f2), this.posY + (adouble2[1] - adouble1[1]) * 1.0D - (double)((f24 + f23) * f15) + 1.5D, this.posZ + (double)((f4 * f23 + f22 * f24) * f2), 0.0F, 0.0F);
                }
            }
        }
    }
    
//    private void updateDragonEnderCrystal()
//    {
//        if (this.healingEnderCrystal != null)
//        {
//            if (this.healingEnderCrystal.dead)
//            {
//                if (!this.worldObj.client)
//                {
//                    this.attackEntityFromPart(this.dragonPartHead, DamageSource.setExplosionSource((Explosion)null), 10);
//                }
//
//                this.healingEnderCrystal = null;
//            }
//            else if (this.ticksExisted % 10 == 0 && this.getHealth() < this.getMaxHealth())
//            {
//                this.setHealth(this.getHealth() + 1);
//            }
//        }
//
//        if (this.rand.chance(10))
//        {
//            float f = 32.0F;
//            List<EntityCrystal> list = this.worldObj.<EntityCrystal>getEntitiesWithinAABB(EntityCrystal.class, this.getEntityBoundingBox().expand((double)f, (double)f, (double)f));
//            EntityCrystal entityendercrystal = null;
//            double d0 = Double.MAX_VALUE;
//
//            for (EntityCrystal entityendercrystal1 : list)
//            {
//                double d1 = entityendercrystal1.getDistanceSqToEntity(this);
//
//                if (d1 < d0)
//                {
//                    d0 = d1;
//                    entityendercrystal = entityendercrystal1;
//                }
//            }
//
//            this.healingEnderCrystal = entityendercrystal;
//        }
//    }
    
    private void collideWithEntities(List<Entity> p_70970_1_)
    {
        double d0 = (this.dragonPartBody.getEntityBoundingBox().minX + this.dragonPartBody.getEntityBoundingBox().maxX) / 2.0D;
        double d1 = (this.dragonPartBody.getEntityBoundingBox().minZ + this.dragonPartBody.getEntityBoundingBox().maxZ) / 2.0D;

        for (Entity entity : p_70970_1_)
        {
            if (entity instanceof EntityLiving)
            {
                double d2 = entity.posX - d0;
                double d3 = entity.posZ - d1;
                double d4 = d2 * d2 + d3 * d3;
                entity.addKnockback(d2 / d4 * 4.0D, 0.20000000298023224D, d3 / d4 * 4.0D);
            }
        }
    }
    
    private void attackEntitiesInList(List<Entity> p_70971_1_)
    {
        for (int i = 0; i < p_70971_1_.size(); ++i)
        {
            Entity entity = (Entity)p_70971_1_.get(i);

            if (entity instanceof EntityLiving)
            {
                entity.attackEntityFrom(DamageSource.causeMobDamage(this), 10);
                this.applyEnchantments(this, entity);
            }
        }
    }

    /**
     * Sets a new target for the flight AI. It can be a random coordinate or a nearby player.
     */
    private void setNewTarget()
    {
        this.forceNewTarget = false;
        List<EntityNPC> list = Lists.newArrayList(this.worldObj.players);
//        Iterator<EntityNPC> iterator = list.iterator();
//
//        while (iterator.hasNext())
//        {
//            if (((EntityNPC)iterator.next()).isSpectator())
//            {
//                iterator.remove();
//            }
//        }

        if (this.rand.chance(2) && !list.isEmpty())
        {
            this.target = this.rand.pick(list);
        }
        else
        {
            while (true)
            {
                this.targetX = 0.0D;
                this.targetY = (double)(70.0F + this.rand.floatv() * 50.0F);
                this.targetZ = 0.0D;
                this.targetX += (double)(this.rand.floatv() * 120.0F - 60.0F);
                this.targetZ += (double)(this.rand.floatv() * 120.0F - 60.0F);
                double d0 = this.posX - this.targetX;
                double d1 = this.posY - this.targetY;
                double d2 = this.posZ - this.targetZ;
                boolean flag = d0 * d0 + d1 * d1 + d2 * d2 > 100.0D;

                if (flag)
                {
                    break;
                }
            }

            this.target = null;
        }
    }
    
    private float simplifyAngle(double p_70973_1_)
    {
        return (float)ExtMath.wrapd(p_70973_1_);
    }

    public boolean attackEntityFromPart(EntityDragonPart dragonPart, DamageSource source, int amount)
    {
        if (dragonPart != this.dragonPartHead)
        {
            amount = amount / 4 + 1;
        }

        float f = this.rotYaw * (float)Math.PI / 180.0F;
        float f1 = ExtMath.sin(f);
        float f2 = ExtMath.cos(f);
        this.targetX = this.posX + (double)(f1 * 5.0F) + (double)((this.rand.floatv() - 0.5F) * 2.0F);
        this.targetY = this.posY + (double)(this.rand.floatv() * 3.0F) + 1.0D;
        this.targetZ = this.posZ - (double)(f2 * 5.0F) + (double)((this.rand.floatv() - 0.5F) * 2.0F);
        this.target = null;

//        if ((source.getEntity() != null && source.getEntity().isPlayer()) || source.isExplosion())
//        {
        this.attackEntityFrom(source, amount);
//        }

        return true;
    }
    
//    public boolean attackEntityFrom(DamageSource source, int amount)
//    {
//        if (source instanceof EntityDamageSource && ((EntityDamageSource)source).getIsThornsDamage())
//        {
//            this.attackDragonFrom(source, amount);
//        }
//
//        return false;
//    }
    
//    protected boolean attackDragonFrom(DamageSource source, int amount)
//    {
//        return super.attackEntityFrom(source, amount);
//    }
    
//    protected void onDeathUpdate()
//    {
//    	super.onDeathUpdate();
//        if (this.deathTime == 20)
//        {
//            float f = (this.rand.floatv() - 0.5F) * 8.0F;
//            float f1 = (this.rand.floatv() - 0.5F) * 4.0F;
//            float f2 = (this.rand.floatv() - 0.5F) * 8.0F;
//            this.worldObj.spawnParticle(EnumParticleTypes.EXPLOSION_HUGE, this.posX + (double)f, this.posY + 2.0D + (double)f1, this.posZ + (double)f2, 0.0D, 0.0D, 0.0D);
//        }
//    }
    
//    private void generatePortal(BlockPos pos)
//    {
//        int i = 4;
//        double d0 = 12.25D;
//        double d1 = 6.25D;
//
//        for (int j = -1; j <= 32; ++j)
//        {
//            for (int k = -4; k <= 4; ++k)
//            {
//                for (int l = -4; l <= 4; ++l)
//                {
//                    double d2 = (double)(k * k + l * l);
//
//                    if (d2 <= 12.25D)
//                    {
//                        BlockPos blockpos = pos.add(k, j, l);
//
//                        if (j < 0)
//                        {
//                            if (d2 <= 6.25D)
//                            {
//                                this.worldObj.setBlockState(blockpos, Blocks.obsidian.getDefaultState());
//                            }
//                        }
//                        else if (j > 0)
//                        {
//                            this.worldObj.setBlockState(blockpos, Blocks.air.getDefaultState());
//                        }
//                        else if (d2 > 6.25D)
//                        {
//                            this.worldObj.setBlockState(blockpos, Blocks.obsidian.getDefaultState());
//                        }
//                        else
//                        {
//                            this.worldObj.setBlockState(blockpos, Blocks.end_portal.getDefaultState());
//                        }
//                    }
//                }
//            }
//        }
//
//        this.worldObj.setBlockState(pos, Blocks.obsidian.getDefaultState());
//        this.worldObj.setBlockState(pos.up(), Blocks.obsidian.getDefaultState());
//        BlockPos blockpos1 = pos.up(2);
//        this.worldObj.setBlockState(blockpos1, Blocks.obsidian.getDefaultState());
//        this.worldObj.setBlockState(blockpos1.west(), Blocks.torch.getDefaultState().withProperty(BlockTorch.FACING, EnumFacing.EAST));
//        this.worldObj.setBlockState(blockpos1.east(), Blocks.torch.getDefaultState().withProperty(BlockTorch.FACING, EnumFacing.WEST));
//        this.worldObj.setBlockState(blockpos1.north(), Blocks.torch.getDefaultState().withProperty(BlockTorch.FACING, EnumFacing.SOUTH));
//        this.worldObj.setBlockState(blockpos1.south(), Blocks.torch.getDefaultState().withProperty(BlockTorch.FACING, EnumFacing.NORTH));
//        this.worldObj.setBlockState(pos.up(3), Blocks.obsidian.getDefaultState());
//        this.worldObj.setBlockState(pos.up(4), Blocks.dragon_egg.getDefaultState());
//    }
    
//    protected void despawnEntity()
//    {
//    }
    
    public Entity[] getParts()
    {
        return this.dragonPartArray;
    }
    
    public boolean canBeCollidedWith()
    {
        return false;
    }

    public World getWorld()
    {
        return this.worldObj;
    }
    
    protected SoundEvent getLivingSound()
    {
        return SoundEvent.DRAGON_IDLE;
    }
    
//    protected Sounds getHurtSound()
//    {
//        return Sounds.MOB_DRAGON_HIT;
//    }
//    
//    protected Sounds getDeathSound()
//    {
//        return Sounds.MOB_DRAGON_HIT;
//    }
    
    protected float getSoundVolume()
    {
        return 5.0F;
    }
    
    public int getTrackingRange() {
    	return 160;
    }
    
    public int getUpdateFrequency() {
    	return 3;
    }
    
    public boolean isSendingVeloUpdates() {
    	return true;
    }
    
	public int getColor() {
		return 0xb900ff;
	}

	public boolean isBoss() {
		return true;
	}

	public boolean getCanSpawnHere() {
		return true;
	}
	
	public Alignment getAlignment() {
		return Alignment.LAWFUL_EVIL;
	}
    
    public EntityType getType() {
    	return EntityType.ANIMAL;
    }
}
