package common.init;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;

import common.block.Block;
import common.block.BlockDoor;
import common.block.BlockLever;
import common.block.BlockLog;
import common.block.BlockPortal;
import common.block.BlockQuartz;
import common.block.BlockRail;
import common.block.BlockRailBase;
import common.block.BlockRailDetector;
import common.block.BlockRailPowered;
import common.block.BlockRotatedPillar;
import common.clipboard.Rotation;
import common.clipboard.RotationValue;
import common.clipboard.Vector;
import common.collect.Lists;
import common.collect.Maps;
import common.properties.IProperty;
import common.properties.PropertyDirection;
import common.util.Facing;
import common.util.Vec3i;
import common.world.State;

public abstract class RotationRegistry {
    private static final Map<Block, Rotation> MAP = new HashMap<Block, Rotation>();
    
    static void register() {
    	for(Block block : common.init.BlockRegistry.REGISTRY) {
        	for(IProperty<?> prop : block.getPropertyMap()) {
        		Predicate<Integer> predicate = null;
        		if(prop == BlockDoor.FACING) {
        			predicate = new Predicate<Integer>() {
						@Override
						public boolean test(Integer meta) {
							return (meta & 8) == 0;
						}
					};
        		}
            	List<RotationValue> values = Lists.newArrayList();
            	Map<Object, Byte> map = Maps.newHashMap();
        		for(int z = 15; z >= 0; z--) {
        			State st = block.getStateFromMeta(z);
        			if(st.getProperties().containsKey(prop)) {
            			map.put(st.getProperties().get(prop), (byte)z);
        			}
        		}

            	byte mask = 0;
        		for(Object v : prop.getAllowedValues()) {
        			if(map.get(v) == null) {
        				continue;
        			}
        			mask |= map.get(v);
        		}
            	if(mask == 0) {
            		continue;
            	}
        		for(Object v : prop.getAllowedValues()) {
        			if(map.get(v) == null) {
        				continue;
        			}
        			Vec3i dv = null;
        			Facing.Axis axis = null;
        			if(prop instanceof PropertyDirection) {
        				dv = ((Facing)v).getDirectionVec();
        			}
        			else if(prop == BlockRotatedPillar.AXIS) {
        				axis = ((Facing.Axis)v);
        			}
        			else if(prop == BlockPortal.AXIS) {
        				axis = ((Facing.Axis)v);
        			}
        			else if(prop == BlockLog.LOG_AXIS) {
        				axis = ((BlockLog.EnumAxis)v).getAxis();
        			}
        			else if(prop == BlockQuartz.VARIANT) {
        				axis = ((BlockQuartz.EnumType)v).getAxis();
        			}
        			else if(prop == BlockLever.FACING) {
        				dv = ((BlockLever.EnumOrientation)v).getFacing().getDirectionVec();
        			}
        			else if(prop == BlockRail.SHAPE || prop == BlockRailDetector.SHAPE || prop == BlockRailPowered.SHAPE) {
        				dv = ((BlockRailBase.EnumRailDirection)v).getFacing().getDirectionVec();
        			}
//        			else if(prop == BlockDoor.HINGE) {
//        				dv = ((BlockDoor.EnumHingePosition)v) == BlockDoor.EnumHingePosition.LEFT ? new Vec3i(-1, 0, 0) : new Vec3i(1, 0, 0);
//        			}
        			if(axis != null) {
	        			switch(axis) {
		    				case X:
		    					dv = new Vec3i(1, 0, 0);
		    					break;
		    				case Y:
		    					dv = new Vec3i(0, 1, 0);
		    					break;
		    				case Z:
		    					dv = new Vec3i(0, 0, 1);
		    					break;
	    				}
        			}
        			if(dv == null) {
        				continue;
        			}
                	values.add(new RotationValue(mask, (byte)(map.get(v) & mask), new Vector(dv.getX(), dv.getY(), dv.getZ())));
        			if(axis != null) {
                    	values.add(new RotationValue(mask, (byte)(map.get(v) & mask), new Vector(-dv.getX(), -dv.getY(), -dv.getZ())));
        			}
        		}
        		if(!values.isEmpty()) {
        			int legacyId = common.init.BlockRegistry.getIdFromBlock(block);
        			Rotation state = new Rotation(values.toArray(new RotationValue[values.size()]), predicate);
//    	    		Log.CONFIG.debug("Block " + game.init.BlockRegistry.REGISTRY.getNameForObject(block) + "/" + legacyId + " mask = " + String.format("0x%x", mask));
//    	    		for(RotationValue value : values) {
//    	    			Log.CONFIG.debug("  meta " + value.data + " -> " + value.direction.toString());
//    	    		}
        			MAP.put(block, state);
        		}
        	}
		}
    }

    
    public static Rotation getRotation(Block block) {
    	return MAP.get(block);
    }
}
