package common.worldgen.feature;

import common.block.Block;
import common.init.Blocks;
import common.rng.Random;
import common.util.BlockPos;
import common.world.WorldServer;
import common.worldgen.FeatureGenerator;

public class WorldGenSand extends FeatureGenerator
{
    private Block block;

    /** The maximum radius used when generating a patch of blocks. */
    private int radius;

    public WorldGenSand(Block p_i45462_1_, int p_i45462_2_)
    {
        this.block = p_i45462_1_;
        this.radius = p_i45462_2_;
    }

    public boolean generate(WorldServer worldIn, Random rand, BlockPos position)
    {
        if (!worldIn.getState(position).getBlock().getMaterial().isColdLiquid())
        {
            return false;
        }
        else
        {
            int i = rand.zrange(this.radius - 2) + 2;
            int j = 2;

            for (int k = position.getX() - i; k <= position.getX() + i; ++k)
            {
                for (int l = position.getZ() - i; l <= position.getZ() + i; ++l)
                {
                    int i1 = k - position.getX();
                    int j1 = l - position.getZ();

                    if (i1 * i1 + j1 * j1 <= i * i)
                    {
                        for (int k1 = position.getY() - j; k1 <= position.getY() + j; ++k1)
                        {
                            BlockPos blockpos = new BlockPos(k, k1, l);
                            Block block = worldIn.getState(blockpos).getBlock();

                            if (block == Blocks.dirt || block == Blocks.grass)
                            {
                                worldIn.setState(blockpos, this.block.getState(), 2);
                            }
                        }
                    }
                }
            }

            return true;
        }
    }
}
