package common.worldgen.tree;

import common.block.Block;
import common.block.BlockLeaves;
import common.init.Blocks;
import common.material.Material;
import common.rng.Random;
import common.util.BlockPos;
import common.world.State;
import common.world.WorldServer;

public abstract class WorldGenHugeTree extends WorldGenTree
{
    /** The base height of the tree */
    protected final int baseHeight;

    /** Sets the metadata for the wood blocks used */
    protected final State woodMetadata;

    /** Sets the metadata for the leaves used in huge trees */
    protected final State leavesMetadata;
    protected int extraRandomHeight;

    public WorldGenHugeTree(boolean notify, int base, int extra, State log, State leaves)
    {
        super(notify);
        this.baseHeight = base;
        this.extraRandomHeight = extra;
        this.woodMetadata = log;
        this.leavesMetadata = leaves.withProperty(BlockLeaves.DECAY, Boolean.valueOf(false));
    }

    protected int func_150533_a(Random p_150533_1_)
    {
        int i = p_150533_1_.zrange(3) + this.baseHeight;

        if (this.extraRandomHeight > 1)
        {
            i += p_150533_1_.zrange(this.extraRandomHeight);
        }

        return i;
    }

    private boolean func_175926_c(WorldServer worldIn, BlockPos p_175926_2_, int p_175926_3_)
    {
        boolean flag = true;

        if (p_175926_2_.getY() >= 1 && p_175926_2_.getY() + p_175926_3_ + 1 <= 512)
        {
            for (int i = 0; i <= 1 + p_175926_3_; ++i)
            {
                int j = 2;

                if (i == 0)
                {
                    j = 1;
                }
                else if (i >= 1 + p_175926_3_ - 2)
                {
                    j = 2;
                }

                for (int k = -j; k <= j && flag; ++k)
                {
                    for (int l = -j; l <= j && flag; ++l)
                    {
                        if (p_175926_2_.getY() + i < 0 || p_175926_2_.getY() + i >= 512 || !this.canBeReplaced(worldIn.getState(p_175926_2_.add(k, i, l)).getBlock()))
                        {
                            flag = false;
                        }
                    }
                }
            }

            return flag;
        }
        else
        {
            return false;
        }
    }

    private boolean func_175927_a(BlockPos p_175927_1_, WorldServer worldIn)
    {
        BlockPos blockpos = p_175927_1_.down();
        Block block = worldIn.getState(blockpos).getBlock();

        if ((block == Blocks.grass || block == Blocks.dirt) && p_175927_1_.getY() >= 2)
        {
            this.setBaseBlock(worldIn, blockpos);
            this.setBaseBlock(worldIn, blockpos.east());
            this.setBaseBlock(worldIn, blockpos.south());
            this.setBaseBlock(worldIn, blockpos.south().east());
            return true;
        }
        else
        {
            return false;
        }
    }

    protected boolean func_175929_a(WorldServer worldIn, Random p_175929_2_, BlockPos p_175929_3_, int p_175929_4_)
    {
        return this.func_175926_c(worldIn, p_175929_3_, p_175929_4_) && this.func_175927_a(p_175929_3_, worldIn);
    }

    protected void func_175925_a(WorldServer worldIn, BlockPos p_175925_2_, int p_175925_3_)
    {
        int i = p_175925_3_ * p_175925_3_;

        for (int j = -p_175925_3_; j <= p_175925_3_ + 1; ++j)
        {
            for (int k = -p_175925_3_; k <= p_175925_3_ + 1; ++k)
            {
                int l = j - 1;
                int i1 = k - 1;

                if (j * j + k * k <= i || l * l + i1 * i1 <= i || j * j + i1 * i1 <= i || l * l + k * k <= i)
                {
                    BlockPos blockpos = p_175925_2_.add(j, 0, k);
                    Material material = worldIn.getState(blockpos).getBlock().getMaterial();

                    if (material == Material.air || material == Material.leaves)
                    {
                        this.setBlockAndNotifyAdequately(worldIn, blockpos, this.leavesMetadata.withProperty(BlockLeaves.TYPE, worldIn.getLeavesGen(blockpos)));
                    }
                }
            }
        }
    }

    protected void func_175928_b(WorldServer worldIn, BlockPos p_175928_2_, int p_175928_3_)
    {
        int i = p_175928_3_ * p_175928_3_;

        for (int j = -p_175928_3_; j <= p_175928_3_; ++j)
        {
            for (int k = -p_175928_3_; k <= p_175928_3_; ++k)
            {
                if (j * j + k * k <= i)
                {
                    BlockPos blockpos = p_175928_2_.add(j, 0, k);
                    Material material = worldIn.getState(blockpos).getBlock().getMaterial();

                    if (material == Material.air || material == Material.leaves)
                    {
                        this.setBlockAndNotifyAdequately(worldIn, blockpos, this.leavesMetadata.withProperty(BlockLeaves.TYPE, worldIn.getLeavesGen(blockpos)));
                    }
                }
            }
        }
    }
}
